CREATE CAST (bigint AS int4) WITH FUNCTION int4(bigint) AS ASSIGNMENT

CREATE CAST (varchar AS citext) WITHOUT FUNCTION AS ASSIGNMENT

CREATE CAST (citext AS varchar) WITHOUT FUNCTION AS IMPLICIT

CREATE CAST (text AS casttesttype) WITHOUT FUNCTION

CREATE CAST (text AS casttesttype) WITHOUT FUNCTION AS IMPLICIT

CREATE CAST (int4 AS casttesttype) WITH INOUT

CREATE CAST (int4 AS casttesttype) WITH FUNCTION int4_casttesttype(int4) AS IMPLICIT
