CREATE SEQUENCE serial START 101

CREATE TEMP SEQUENCE serial

CREATE TEMP SEQUENCE serial NO CYCLE

CREATE SEQUENCE IF NOT EXISTS "SomeSchema"."SomeSerial" as smallint

CREATE TEMP SEQUENCE serial INCREMENT 2 MINVALUE 5 NO MAXVALUE

CREATE TEMP SEQUENCE serial INCREMENT 2 MINVALUE 5 MAXVALUE 15 START WITH 7 CYCLE

CREATE SEQUENCE serial OWNED BY "SomeSchema"."SomeTable"."SomeColumn"
