ALTER TEXT SEARCH CONFIGURATION my_config
  ALTER MAPPING FOR word, asciiword REPLACE english WITH swedish

ALTER TEXT SEARCH CONFIGURATION my_config ADD MAPPING FOR word WITH english

ALTER TEXT SEARCH CONFIGURATION my_config DROP MAPPING IF EXISTS FOR word
