ALTER TYPE t1 ADD VALUE 'value3'

ALTER TYPE t1 ADD VALUE 'value2.5' BEFORE 'value3'

ALTER TYPE t1 ADD VALUE 'value3.5' AFTER 'value3'

ALTER TYPE t1 ADD VALUE IF NOT EXISTS 'value3'

ALTER TYPE t1 RENAME VALUE 'value1' TO 'value2'
