# -*- coding: utf-8 -*-
# :Project:   pglast -- Top level error exception class
# :Created:   mer 02 ago 2017 15:35:39 CEST
# :Author:    Lele Gaifax <lele@metapensiero.it>
# :License:   GNU General Public License version 3 or later
# :Copyright: © 2017, 2018 Lele Gaifax
#

class Error(Exception):
    "Top level error exception."
