SET CONSTRAINTS ALL IMMEDIATE

SET CONSTRAINTS foo, bar IMMEDIATE

SET CONSTRAINTS ALL DEFERRED

SET CONSTRAINTS foo, bar DEFERRED

SET TIME ZONE LOCAL

SET LOCAL TIME ZONE DEFAULT

SET SESSION TIME ZONE 'Europe/Rome'

SET datestyle TO postgres, dmy

SET search_path TO my_schema, public

SET TRANSACTION ISOLATION LEVEL READ COMMITTED

SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

SET TRANSACTION ISOLATION LEVEL REPEATABLE READ

SET TRANSACTION ISOLATION LEVEL SERIALIZABLE

SET TRANSACTION NOT DEFERRABLE

SET TRANSACTION SNAPSHOT 'foo'
