create schema myschema

create schema authorization joe

create schema if not exists test authorization joe

create schema test authorization public

create schema test authorization current_user

create schema test authorization session_user
