ALTER DEFAULT PRIVILEGES FOR ROLE role1, role2 IN SCHEMA s1,  s2
GRANT ALL PRIVILEGES ON TABLES TO grantee1, grantee2

ALTER DEFAULT PRIVILEGES FOR ROLE role1, role2 IN SCHEMA s1,  s2
REVOKE INSERT, UPDATE ON TABLES FROM grantee1, grantee2
