CREATE OR REPLACE LANGUAGE lang
=
CREATE EXTENSION IF NOT EXISTS lang

CREATE OR REPLACE TRUSTED LANGUAGE trustlang HANDLER s1.fhandler INLINE s1.finline VALIDATOR fvalidator
=
CREATE OR REPLACE TRUSTED PROCEDURAL LANGUAGE trustlang
  HANDLER s1.fhandler
  INLINE s1.finline
  VALIDATOR fvalidator
